﻿using System;
using Metal;

namespace VixarEngine
{
    public class MetalModel3D
    {
        public float[] _vertexData;
		IMTLBuffer _vertexBuffer;
		nuint _size;
        protected string _textureName;
        public Texture _texture;
        protected float[] _matriciesData;
        protected int _maxBonesSize;
		IMTLBuffer _boneBuffer;
        protected int _animationSize;
        float _time = 0;

		public MetalModel3D()
        {
        }

        // Designated initializer
		public bool Load3DData(IMTLDevice device,string label)
		{
            if (device == null)
            {
                throw new Exception("ERROR: Invalid device!");
            }
			_vertexBuffer = device.CreateBuffer<float>(_vertexData, (MTLResourceOptions)0);
            if (_vertexBuffer == null)
            {
                Console.WriteLine("ERROR: Failed creating a vertex buffer!");
                return false;
            }
			_vertexBuffer.Label = label + " Vertices";
			_size = (nuint)(_vertexData.Length / 6);
            if (_maxBonesSize > 0)
            {
				_animationSize = _matriciesData.Length / (_maxBonesSize * 16);
            }
            else
            {
                _matriciesData = new float[16] { 1,0,0,0, 0,1,0,0, 0,0,1,0, 0,0,0,1};
				_animationSize = 0; 
            }
            _boneBuffer = device.CreateBuffer<float>(_matriciesData, MTLResourceOptions.CpuCacheModeDefault);
            if (_boneBuffer == null)
            {
                Console.WriteLine("ERROR: Failed creating a bone buffer!");
                return false;
            }
            _boneBuffer.Label = label + " Bones";
			return LoadTexture(device, label);
		}

        public bool LoadTexture(IMTLDevice device,string label)
        {
            _texture = new Texture(_textureName);
			bool isAcquired = _texture.Finalize(device);
			if (!isAcquired)
			{
				Console.WriteLine("ERROR: Failed creating an input 2d texture!");
				return false;
			}
			_texture.MetalTexture.Label = label+" Texture";

            return true;
		}

        public void Encode(IMTLRenderCommandEncoder renderEncoder)
        {
			renderEncoder.SetFragmentTexture(_texture.MetalTexture, 0);
/*
            MTLSamplerDescriptor samplerDescriptor = new MTLSamplerDescriptor();
            samplerDescriptor.MinFilter = MTLSamplerMinMagFilter.Nearest;
            samplerDescriptor.MagFilter = MTLSamplerMinMagFilter.Linear;
            samplerDescriptor.SAddressMode = MTLSamplerAddressMode.Repeat;
            samplerDescriptor.TAddressMode = MTLSamplerAddressMode.Repeat;

            IMTLSamplerState sampler = device.CreateSamplerState(samplerDescriptor);
            renderEncoder.SetFragmentSamplerState(sampler,0);
*/
            renderEncoder.SetVertexBuffer(_vertexBuffer, 0, 0);
			renderEncoder.SetVertexBuffer(_boneBuffer, (uint)(_maxBonesSize * 16 * (int)(_time) * sizeof(float)), 1);
			_time += 30.0f / 60.0f;
			if (_time >= _animationSize) _time = 0;
			// tell the render context we want to draw our primitives
			renderEncoder.DrawPrimitives(MTLPrimitiveType.Triangle, 0, _size, 1);
        }
	}
}
