#include <metal_graphics>
#include <metal_matrix>
#include <metal_geometric>
#include <metal_math>
#include <metal_texture>

using namespace metal;

typedef struct
{
	packed_float3 position;
	packed_float2 uv;
	float bone;
} vertex_t;
typedef struct
{
	float4 bone0;
	float4 bone1;
	float4 bone2;
	float4 bone3;
} bones;
struct VertexInOut
{
    float4 m_Position [[position]];
    float2 m_TexCoord [[user(texturecoord)]];
};

vertex VertexInOut texturedQuadVertex(device vertex_t*   vertex_array [[ buffer(0) ]],
									  device bones*      bone_array   [[ buffer(1) ]],
									  constant float4x4 *pMVP         [[ buffer(2) ]],
                                      uint               vid          [[ vertex_id ]])
{
    VertexInOut outVertices;
	
	float4 pos = float4(float3(vertex_array[vid].position),1.0);
	int boneId = int(vertex_array[vid].bone);
	if (boneId >= 0)
	{
		float4 bone0 = bone_array[boneId].bone0;
		float4 bone1 = bone_array[boneId].bone1;
		float4 bone2 = bone_array[boneId].bone2;
		float4 bone3 = bone_array[boneId].bone3;
		float4x4 bone = float4x4(bone0,bone1,bone2,bone3);
		outVertices.m_Position = *pMVP * bone * pos;
	}
	else
	{
		outVertices.m_Position = *pMVP * pos;
	}
    outVertices.m_TexCoord = vertex_array[vid].uv;
    
    return outVertices;
}

fragment half4 texturedQuadFragment(VertexInOut     inFrag    [[ stage_in ]],
                                    texture2d<half>  tex2D     [[ texture(0) ]])
{
    constexpr sampler quad_sampler;
    half4 color = tex2D.sample(quad_sampler, inFrag.m_TexCoord);
    
    return color;
}
